@@ -1,5 +1,5 @@
 ; AisleRiot - sol.scm
-; Copyright (C) 1998 Jonathan Blandford <jrb@mit.edu>
+; Copyright (C) 1998, 2003 Jonathan Blandford <jrb@mit.edu>
 ;
 ; This game is free software; you can redistribute it and/or modify
 ; it under the terms of the GNU General Public License as published by
@@ -16,6 +16,16 @@
 ; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 ; USA
 
+(use-modules (ice-9 format))
+
+;; Compatibility code so we run on both guile 1.3 and 1.5
+
+(define (eval-1 code)
+  (if (string<? (version) "1.5")
+      (eval code)
+      (eval code (current-module))))
+        
+
 ;; Constants:
 
 (define jack 11)
@@ -39,21 +49,36 @@
 
 (define DECK '())
 
+; The list of variables to save when saving the game state
+(define variable-list '())
+
 ;; NEW-GAME PROCEDURES
 ; these may be used in game files during the new-game procedure.
 
 ; This procedure MUST be called at the start of the new-game procedure.
+;
+; Note that variable-list is not cleared, this is because defines are normally
+; done before calling this and we would loose our variable list. At worst
+; case we end up saving and restoring variables that are not currently in use 
+; (but will be defined) so it will work out OK.
 (define (initialize-playing-area)
   (reset-surface)
   (set! FLIP-COUNTER 0)
   (set! SLOTS 0)
-  (set! HORIZPOS (get-horiz-start))
-  (set! VERTPOS (get-vert-start))
+  (set! HORIZPOS 0)
+  (set! VERTPOS 0)
   (set! IN-GAME #f)
   (set! MOVE '())
-  (set-statusbar-message "")
+  (set-statusbar-message " ")
   (set! HISTORY '()))
 
+; Use this instead of define for variables which determine the state of
+; the game. i.e. anything that isn't a constant. This is so undo/redo
+; is transparent. It should behave otherwise identically to define.
+(defmacro def-save-var (nm value)
+  `(begin (define ,nm ,value)
+          (set! variable-list (cons ',nm variable-list))))
+
 ; create a 52 card deck (puts list of cards into DECK)
 (define (make-standard-deck)
   (if (= ace 14)
@@ -239,7 +264,7 @@
 	((eq? (get-suit card) spade) black)
 	((eq? (get-suit card) heart) red)
 	((eq? (get-suit card) diamond) red)
-	(#t "Unknown color")))
+	(#t (_"Unknown color"))))
 
 (define (get-value card) 
   (car card))
@@ -303,39 +328,98 @@
 (define (display-list . objs)
   (map display objs) (newline))
 
-; hint procedures: these will be superceeded shortly.
+; hint procedures
 (define (get-value-name value)
-  (cond ((eq? value ace) "ace")
-	((eq? value 2) "two")
-	((eq? value 3) "three")
-	((eq? value 4) "four")
-	((eq? value 5) "five")
-	((eq? value 6) "six")
-	((eq? value 7) "seven")
-	((eq? value 8) "eight")
-	((eq? value 9) "nine")
-	((eq? value 10) "ten")
-	((eq? value jack) "jack")
-	((eq? value queen) "queen")
-	((eq? value king) "king")
-	(#t "Unknown value")))
+  (cond ((eq? value ace) (_"ace"))
+        ((eq? value 2) (_"two"))
+        ((eq? value 3) (_"three"))
+        ((eq? value 4) (_"four"))
+        ((eq? value 5) (_"five"))
+        ((eq? value 6) (_"six"))
+        ((eq? value 7) (_"seven"))
+        ((eq? value 8) (_"eight"))
+        ((eq? value 9) (_"nine"))
+        ((eq? value 10) (_"ten"))
+        ((eq? value jack) (_"jack"))
+        ((eq? value queen) (_"queen"))
+        ((eq? value king) (_"king"))
+        (#t (_"Unknown value"))))
 
 (define (get-suit-name suit)
-  (cond ((eq? suit club) "clubs")
-	((eq? suit spade) "spades")
-	((eq? suit heart) "hearts")
-	((eq? suit diamond) "diamonds")
-	(#t "Unknown suit")))
+  (cond ((eq? suit club) (_"clubs"))
+        ((eq? suit spade) (_"spades"))
+        ((eq? suit heart) (_"hearts"))
+        ((eq? suit diamond) (_"diamonds"))
+        (#t (_"Unknown suit"))))
 
 (define (get-joker-name card) 
-  (if (is-black? card) "black joker" "red joker"))
+  (if (is-black? card) (_"black joker") (_"red joker")))
 
 (define (get-name card)
-  (if (is-joker? card)
-      (get-joker-name card)
-      (string-append (get-value-name (get-value card)) 
-		     " of "
-		     (get-suit-name (get-suit card)))))
+  (let ((value (get-value card)) (suit (get-suit card)))
+    (if (is-joker? card)
+        (get-joker-name card)
+        (cond ((eq? suit club) 
+               (cond ((eq? value ace) (_"the ace of clubs"))
+                     ((eq? value 2) (_"the two of clubs"))
+                     ((eq? value 3) (_"the three of clubs"))
+                     ((eq? value 4) (_"the four of clubs"))
+                     ((eq? value 5) (_"the five of clubs"))
+                     ((eq? value 6) (_"the six of clubs"))
+                     ((eq? value 7) (_"the seven of clubs"))
+                     ((eq? value 8) (_"the eight of clubs"))
+                     ((eq? value 9) (_"the nine of clubs"))
+                     ((eq? value 10) (_"the ten of clubs"))
+                     ((eq? value jack) (_"the jack of clubs"))
+                     ((eq? value queen) (_"the queen of clubs"))
+                     ((eq? value king) (_"the king of clubs"))
+                     (#t (_"the unknown card"))))
+              ((eq? suit spade) 
+               (cond ((eq? value ace) (_"the ace of spades"))
+                     ((eq? value 2) (_"the two of spades"))
+                     ((eq? value 3) (_"the three of spades"))
+                     ((eq? value 4) (_"the four of spades"))
+                     ((eq? value 5) (_"the five of spades"))
+                     ((eq? value 6) (_"the six of spades"))
+                     ((eq? value 7) (_"the seven of spades"))
+                     ((eq? value 8) (_"the eight of spades"))
+                     ((eq? value 9) (_"the nine of spades"))
+                     ((eq? value 10) (_"the ten of spades"))
+                     ((eq? value jack) (_"the jack of spades"))
+                     ((eq? value queen) (_"the queen of spades"))
+                     ((eq? value king) (_"the king of spades"))
+                     (#t (_"the unknown card"))))
+              ((eq? suit heart) 
+               (cond ((eq? value ace) (_"the ace of hearts"))
+                     ((eq? value 2) (_"the two of hearts"))
+                     ((eq? value 3) (_"the three of hearts"))
+                     ((eq? value 4) (_"the four of hearts"))
+                     ((eq? value 5) (_"the five of hearts"))
+                     ((eq? value 6) (_"the six of hearts"))
+                     ((eq? value 7) (_"the seven of hearts"))
+                     ((eq? value 8) (_"the eight of hearts"))
+                     ((eq? value 9) (_"the nine of hearts"))
+                     ((eq? value 10) (_"the ten of hearts"))
+                     ((eq? value jack) (_"the jack of hearts"))
+                     ((eq? value queen) (_"the queen of hearts"))
+                     ((eq? value king) (_"the king of hearts"))
+                     (#t (_"the unknown card"))))
+              ((eq? suit diamond) 
+               (cond ((eq? value ace) (_"the ace of diamonds"))
+                     ((eq? value 2) (_"the two of diamonds"))
+                     ((eq? value 3) (_"the three of diamonds"))
+                     ((eq? value 4) (_"the four of diamonds"))
+                     ((eq? value 5) (_"the five of diamonds"))
+                     ((eq? value 6) (_"the six of diamonds"))
+                     ((eq? value 7) (_"the seven of diamonds"))
+                     ((eq? value 8) (_"the eight of diamonds"))
+                     ((eq? value 9) (_"the nine of diamonds"))
+                     ((eq? value 10) (_"the ten of diamonds"))
+                     ((eq? value jack) (_"the jack of diamonds"))
+                     ((eq? value queen) (_"the queen of diamonds"))
+                     ((eq? value king) (_"the king of diamonds"))
+                     (#t (_"the unknown card"))))
+              (#t (_"the unknown card"))))))
 
 (define (move-n-cards! start-slot end-slot cards)
   (add-cards! end-slot cards))
@@ -399,12 +483,12 @@
 
 (define (get-and-increment-position)
   (let ((retval (list HORIZPOS VERTPOS)))
-    (set! HORIZPOS (+ HORIZPOS (get-horiz-offset)))
+    (set! HORIZPOS (+ HORIZPOS 1))
     retval))
 
 (define (linefeed-position)
-  (set! HORIZPOS (get-horiz-start))
-  (set! VERTPOS (+ VERTPOS (get-vert-offset))))
+  (set! HORIZPOS 0)
+  (set! VERTPOS (+ VERTPOS 1)))
 
 (define (register-undo-function function data)
   (set! MOVE (cons '(function data) (cdr MOVE))))
@@ -435,7 +519,13 @@
       (begin
 	(set! HISTORY (cons MOVE HISTORY))
 	(set! FUTURE '())
-	(set! MOVE '()))))
+	(set! MOVE '())
+        (if (null? HISTORY)
+            (undo-set-sensitive #f)
+            (undo-set-sensitive #t))
+        (if (null? FUTURE)
+            (redo-set-sensitive #f)
+            (redo-set-sensitive #t)))))
 
 (define (return-cards card-positions slot-id)
   (and (not (= 0 (length card-positions)))
@@ -457,7 +547,10 @@
        (eval-move (car HISTORY))
        (set! FUTURE (cons MOVE FUTURE))
        (set! HISTORY (cdr HISTORY))
-       (set! MOVE '())))
+       (set! MOVE '())
+       (redo-set-sensitive #t)
+       (if (null? HISTORY)
+           (undo-set-sensitive #f))))
 
 ; called from C:
 (define (redo)
@@ -466,11 +559,15 @@
        (eval-move (car FUTURE))
        (set! HISTORY (cons MOVE HISTORY))
        (set! FUTURE (cdr FUTURE))
-       (set! MOVE '())))
+       (set! MOVE '())
+       (undo-set-sensitive #t)
+       (if (null? FUTURE)
+           (redo-set-sensitive #f))))
 
 (define (undo-func data)
   (set-score! (car data))
-  (set! FLIP-COUNTER (cadr data)))
+  (set! FLIP-COUNTER (cadr data))
+  (restore-variables variable-list (caddr data)))
 ;(register-undo-function undo-func '(score FLIP-COUNTER))
 	     
 (define (snapshot-board slot-id moving-slot old-cards)
@@ -486,9 +583,31 @@
 ; called from C:
 (define (record-move slot-id old-cards)
   (set! MOVE (list undo-func 
-		   (list (get-score) FLIP-COUNTER)
-		   (snapshot-board 0 slot-id old-cards))))
+                   (list (get-score) FLIP-COUNTER 
+                         (save-variables variable-list))
+                   (snapshot-board 0 slot-id old-cards))))
 
 ; called from C:
 (define (discard-move)
   (set! MOVE '()))
+
+;; Routines for saving/restoring generic variables
+
+; Get a list of values for the variables we wish to save.
+(define save-variables
+  (lambda (names)
+    (if (equal? '() names)
+        '()
+        (cons (eval-1 (car names)) 
+              (save-variables (cdr names))))))
+
+; Restore all the state variables for a game
+(define restore-variables
+  (lambda (names values)
+    (or (equal? '() names)
+        (begin
+          (eval-1 (list 'set! (car names) (car values)))
+          (restore-variables (cdr names) (cdr values))
+          ))))
+
+
