@@ -23,14 +23,13 @@
 #include <config.h>
 #include <gnome.h>
 
-#define PIXMAPWIDTH 10
-#define PIXMAPHEIGHT 10
 #define BOARDWIDTH 92
 #define BOARDHEIGHT 66
 #define BLANKPIXMAP 0
 
 #define NUMWORMS 4
 
+#define NUM_COLORS 7
 #define WORMRED 12
 #define WORMGREEN 13
 #define WORMBLUE 14
@@ -56,11 +55,11 @@ void gnibbles_draw_big_pixmap (gint which, gint x, gint y);
 void gnibbles_draw_pixmap_buffer (gint which, gint x, gint y);
 void gnibbles_draw_big_pixmap_buffer (gint which, gint x, gint y);
 void gnibbles_load_pixmap (GtkWidget *window);
-void gnibbles_load_level (GtkWidget *window, int level);
-void gnibbles_init ();
-void gnibbles_add_bonus (int regular);
-void gnibbles_destroy ();
-gint gnibbles_move_worms ();
+void gnibbles_load_logo (GtkWidget *window);
+void gnibbles_load_level (GtkWidget *window, gint level);
+void gnibbles_init (void);
+void gnibbles_add_bonus (gint regular);
+gint gnibbles_move_worms (void);
 gint gnibbles_keypress_worms (guint keyval);
 void gnibbles_undraw_worms (gint data);
 void gnibbles_play_sound (const char *which);
